<?php

declare(strict_types=1);

namespace Drupal\checklistapi\Entity;

use Drupal\Core\Config\Action\Attribute\ActionMethod;
use Drupal\Core\Config\Entity\ConfigEntityBundleBase;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\checklistapi\ChecklistInterface;

/**
 * Defines the checklist configuration entity.
 *
 * @ConfigEntityType(
 *   id = "checklist",
 *   label = @Translation("Checklist"),
 *   label_collection = @Translation("Checklists"),
 *   label_singular = @Translation("checklist"),
 *   label_plural = @Translation("checklists"),
 *   label_count = @PluralTranslation(
 *     singular = "@count checklist",
 *     plural = "@count checklists"
 *   ),
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "description",
 *     "help",
 *     "task_groups",
 *   },
 *   constraints = {
 *     "ImmutableProperties" = {"id"}
 *   },
 *   bundle_of = "checklist_task",
 * )
 */
final class Checklist extends ConfigEntityBundleBase implements ChecklistInterface {

  /**
   * The checklist's ID (machine name).
   *
   * @var string
   */
  protected string $id;

  /**
   * The checklist's human-readable name.
   *
   * @var string
   */
  protected string $label;

  /**
   * A short, administrative description of the checklist.
   *
   * @var string
   */
  protected string $description = '';

  /**
   * Help text or instructions to display with this checklist.
   *
   * @var string
   */
  protected string $help = '';

  /**
   * The groups under which tasks can be, well, grouped.
   *
   * @var array[]
   */
  protected array $task_groups = [];

  /**
   * Sets the human-readable label of this checklist.
   *
   * @param string $label
   *   The new label.
   *
   * @return $this
   */
  #[ActionMethod(adminLabel: new TranslatableMarkup('Set label'), pluralize: FALSE)]
  public function setLabel(string $label): self {
    $property = $this->getEntityType()->getKey('label');
    assert(is_string($property));
    return $this->set($property, $label);
  }

  /**
   * {@inheritdoc}
   */
  public function getHelpText(): string {
    return $this->get('help');
  }

  /**
   * Sets the help text (or instructions) to display with this checklist.
   *
   * @param string $help
   *   The new help text.
   *
   * @return $this
   */
  #[ActionMethod(adminLabel: new TranslatableMarkup('Set help text'), pluralize: FALSE)]
  public function setHelpText(string $help): self {
    return $this->set('help', $help);
  }

  /**
   * {@inheritdoc}
   */
  public function getTaskGroups(): array {
    return $this->get('task_groups');
  }

  /**
   * {@inheritdoc}
   */
  #[ActionMethod(adminLabel: new TranslatableMarkup('Add task group'))]
  public function addTaskGroup(string $key, string $title, string $description = '', int $weight = 0): self {
    $groups = $this->getTaskGroups();

    $groups[$key] = [
      'title' => $title,
      'description' => $description,
      'weight' => $weight,
    ];
    return $this->set('task_groups', $groups);
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription(): string {
    return $this->get('description');
  }

  /**
   * {@inheritdoc}
   */
  #[ActionMethod(adminLabel: new TranslatableMarkup('Set description'), pluralize: FALSE)]
  public function setDescription($description): self {
    return $this->set('description', (string) $description);
  }

}
