<?php

declare(strict_types=1);

namespace Drupal\checklistapi\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;

/**
 * Defines a task in a checklist.
 *
 * @ContentEntityType(
 *   id = "checklist_task",
 *   label = @Translation("Task"),
 *   label_collection = @Translation("Tasks"),
 *   label_singular = @Translation("task"),
 *   label_plural = @Translation("tasks"),
 *   label_count = @PluralTranslation(
 *     singular = "@count task",
 *     plural = "@count tasks"
 *   ),
 *   bundle_label = @Translation("Checklist"),
 *   base_table = "checklist_task",
 *   data_table = "checklist_task_data",
 *   translatable = TRUE,
 *   entity_keys = {
 *     "id" = "id",
 *     "bundle" = "checklist",
 *     "label" = "title",
 *     "langcode" = "langcode",
 *     "uuid" = "uuid",
 *   },
 *   bundle_entity_type = "checklist"
 * )
 */
final class Task extends ContentEntityBase {

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {
    $definitions = parent::baseFieldDefinitions($entity_type);

    $label = $entity_type->getKey('label');
    $definitions[$label] = BaseFieldDefinition::create('string')
      ->setLabel(t('Title'))
      ->setDescription(t('The human-readable name of the task.'))
      ->setCardinality(1)
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayConfigurable('form', TRUE)
      ->setRequired(TRUE)
      ->setTranslatable(TRUE);

    $definitions['description'] = BaseFieldDefinition::create('text_long')
      ->setLabel(t('Description'))
      ->setDescription(t('A description of this task, optionally including instructions for completing it.'))
      ->setCardinality(1)
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayConfigurable('form', TRUE)
      ->setTranslatable(TRUE);

    $definitions['group'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Group'))
      ->setDescription(t('The ID of the group in the checklist this task belongs to.'))
      ->setCardinality(1)
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayConfigurable('form', TRUE);

    $definitions['weight'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Weight'))
      ->setDescription(t('The weight of this task, relative to others in the same group.'))
      ->setCardinality(1)
      ->setRequired(TRUE)
      ->setDefaultValue(0);

    return $definitions;
  }

}
